/// pptToOPML
/// 
/// Purpose: Convert PowerPoint documents to OPML format, using the Hyperscope OPML extensions
///    documented at http://www.hyperscope.org
///
/// Description: Extracts TEXT from Microsoft PowerPoint, equivalent to viewing all levels in powerpoint's Outline
//     mode.  Also includes notes for each slide, if present, and has an option to include slide images as well.
/// 
/// Requires: Microsoft PowrePoint v 11.0 (or greater) to be installed on the machine
///
/// 
/// Author: Adam Cheyer (adam.cheyer@sri.com)
/// Date: 09/08/06
/// 
/// License: LGPL
/// 

// Imports
using System;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using System.IO;

namespace pptToOPML
{
	/// <summary>
	/// frmMain is the primary GUI for pptToOPML application
	/// </summary>
	public class frmMain : System.Windows.Forms.Form
	{

        /// <summary>
        /// Class variables
        /// </summary>
        private int parNum = 1;
        private int parNums = 0;
        XmlTextWriter outfil;

        /// <summary>
		/// User Interface Objects
		/// </summary>
		private System.Windows.Forms.MenuItem mnuFile;
		private System.Windows.Forms.MenuItem mnuExit;
        private System.Windows.Forms.MainMenu mainMenu;

        private IContainer components;
        private System.Windows.Forms.OpenFileDialog openFileDialog;
        private ProgressBar progressBar1;
        private MenuItem mnuAbout;
        private MenuItem mnuOptions;
        private MenuItem mnuSaveJPGs;
        private MenuItem mnuIncludeNotes;
        private Button btnLoadSource;

		public frmMain()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(frmMain));
            this.mainMenu = new System.Windows.Forms.MainMenu(this.components);
            this.mnuFile = new System.Windows.Forms.MenuItem();
            this.mnuExit = new System.Windows.Forms.MenuItem();
            this.mnuOptions = new System.Windows.Forms.MenuItem();
            this.mnuSaveJPGs = new System.Windows.Forms.MenuItem();
            this.mnuAbout = new System.Windows.Forms.MenuItem();
            this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
            this.progressBar1 = new System.Windows.Forms.ProgressBar();
            this.btnLoadSource = new System.Windows.Forms.Button();
            this.mnuIncludeNotes = new System.Windows.Forms.MenuItem();
            this.SuspendLayout();
            // 
            // mainMenu
            // 
            this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuFile,
            this.mnuOptions,
            this.mnuAbout});
            // 
            // mnuFile
            // 
            this.mnuFile.Index = 0;
            this.mnuFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuExit});
            this.mnuFile.Text = "&File";
            // 
            // mnuExit
            // 
            this.mnuExit.Index = 0;
            this.mnuExit.Text = "E&xit";
            this.mnuExit.Click += new System.EventHandler(this.mnuExit_Click);
            // 
            // mnuOptions
            // 
            this.mnuOptions.Index = 1;
            this.mnuOptions.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.mnuSaveJPGs,
            this.mnuIncludeNotes});
            this.mnuOptions.Text = "&Options";
            // 
            // mnuSaveJPGs
            // 
            this.mnuSaveJPGs.Checked = true;
            this.mnuSaveJPGs.Index = 0;
            this.mnuSaveJPGs.Text = "&Save JPGs";
            this.mnuSaveJPGs.Click += new System.EventHandler(this.mnuSaveJPGs_Click);
            // 
            // mnuAbout
            // 
            this.mnuAbout.Index = 2;
            this.mnuAbout.Text = "&About";
            this.mnuAbout.Click += new System.EventHandler(this.mnuAbout_Click);
            // 
            // openFileDialog
            // 
            this.openFileDialog.Multiselect = true;
            this.openFileDialog.Title = "SAMS Parser.Net";
            // 
            // progressBar1
            // 
            this.progressBar1.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.progressBar1.Location = new System.Drawing.Point(0, 70);
            this.progressBar1.Name = "progressBar1";
            this.progressBar1.Size = new System.Drawing.Size(236, 28);
            this.progressBar1.TabIndex = 1;
            // 
            // btnLoadSource
            // 
            this.btnLoadSource.Dock = System.Windows.Forms.DockStyle.Fill;
            this.btnLoadSource.Location = new System.Drawing.Point(0, 0);
            this.btnLoadSource.Name = "btnLoadSource";
            this.btnLoadSource.Size = new System.Drawing.Size(236, 70);
            this.btnLoadSource.TabIndex = 2;
            this.btnLoadSource.Text = "Load PPT File";
            this.btnLoadSource.UseVisualStyleBackColor = true;
            this.btnLoadSource.Click += new System.EventHandler(this.btnLoadSource_Click);
            // 
            // mnuIncludeNotes
            // 
            this.mnuIncludeNotes.Checked = true;
            this.mnuIncludeNotes.Index = 1;
            this.mnuIncludeNotes.Text = "Include &Notes";
            this.mnuIncludeNotes.Click += new System.EventHandler(this.mnuIncludeNotes_Click);
            // 
            // frmMain
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(236, 98);
            this.Controls.Add(this.btnLoadSource);
            this.Controls.Add(this.progressBar1);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Menu = this.mainMenu;
            this.Name = "frmMain";
            this.Text = "PowerPoint To OPML";
            this.Load += new System.EventHandler(this.frmMain_Load);
            this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain());
		}

        /// <summary>
        /// Returns false for illegal characters
        /// </summary>
        /// <param name="c"></param>
        /// <returns></returns>
        private bool OKChar(char c)
        {
            bool ok = ((c >= ' ') && (c <= 'z'));
            string  bad = "<>&";
            if (bad.IndexOf(c) >= 0)
                ok = false;
            return ok;
        }

        // Cleans string to only legal characters
        private string CleanString(string s)
        {
            string cs = "";
            for (int i = 0; i < s.Length; i++)
               if (OKChar(s[i]))
                  cs = cs + s[i];
            return cs;
        }

        /// <summary>
        /// Gets the title for the input slide.  If none found, returns "Slide" + slideNum
        /// </summary>
        /// <param name="sld"></param>
        /// <param name="slideNum"></param>
        /// <returns></returns>
        private string GetTitle(Microsoft.Office.Interop.PowerPoint.Slide sld, int slideNum)
        {
            if (sld.Shapes.HasTitle == Microsoft.Office.Core.MsoTriState.msoTrue)
            {
                string title = CleanString(sld.Shapes.Title.TextFrame.TextRange.Text.Trim());
                if (title != "")
                    return title;
                else
                    return "Slide" + slideNum.ToString();
            }
            else
                return "Slide" + slideNum.ToString();            
        }



        /// <summary>
        /// Recursive function to nest outline elements on a slide.
        /// </summary>
        /// <param name="shp">A placeholder shape containing multiple paragraphs</param>
        /// <param name="slideId">ID of the slide number, for use generating the NIDs</param>
        private void writeSection(Microsoft.Office.Interop.PowerPoint.Shape shp, int slideId)
        {
            bool done = false;
            bool writeClose = false;


            Microsoft.Office.Interop.PowerPoint.TextRange tr;

            // Get the paragraph number in global var parNum
            tr = shp.TextFrame.TextRange.Paragraphs(parNum, 1);

            // Write it out.  It may need to contain subparagraphs (recursive) so don't close yet.
            outfil.WriteStartElement("outline");
            outfil.WriteAttributeString("text", "<h2>" + CleanString(tr.Text.Trim()) + "</h2>");
            outfil.WriteAttributeString("hs:nid", "0" + slideId.ToString() + parNum.ToString());
            writeClose = true;

            int currentLevel = tr.IndentLevel;

            parNum = parNum + 1;

            // Loop through all other paragraphs in the placeholder.
            //  If a deeper level than the current, recursively drill into it
            //  If same, close the previous level and print the new level
            //  If the next level is higher up than we are, exit recursive function
            while ((parNum <= parNums) && !done)
            {
                tr = shp.TextFrame.TextRange.Paragraphs(parNum, 1);

                if (tr.IndentLevel == currentLevel)
                {
                    if (writeClose)
                    {
                        outfil.WriteEndElement();
                        writeClose = false;
                    }
                    string text = CleanString(tr.Text.Trim());

                    if (!text.Equals(""))
                    {
                        outfil.WriteStartElement("outline");
                        outfil.WriteAttributeString("text", "<h2>" + text + "</h2>");
                        outfil.WriteAttributeString("hs:nid", "0" + slideId.ToString() + parNum.ToString());
                        writeClose = true;
                    }
                    parNum = parNum + 1;

                }
                else if (tr.IndentLevel > currentLevel)
                {
                    writeSection(shp, slideId);
                }
                else
                {
                    done = true;
                }
            }

            if (writeClose)
            {
                outfil.WriteEndElement();
                writeClose = false;
            }
        }

        /// <summary>
        /// Lookup the property for a document.  
        /// Properties may be: 
        ///   "Title", "Subject", "Author", "Keywords", "Revision Number", 
        ///   "Creation Date", "Last Save Time"
        /// </summary>
        /// <param name="documentProperties">List of document properties</param>
        /// <param name="propName">Name of property to return</param>
        /// <returns>Property value</returns>
        private string GetDocumentProperty(object documentProperties, string propName)
        {
            string propValue = "";
            try
            {
                Type propertyType = documentProperties.GetType();

                object property = propertyType.InvokeMember("Item",
                        System.Reflection.BindingFlags.Default |
                        System.Reflection.BindingFlags.GetProperty,
                        null,
                        documentProperties,
                        new object[] { propName });

                Type validatedType = property.GetType();

                propValue = validatedType.InvokeMember("Value",
                    System.Reflection.BindingFlags.Default |
                    System.Reflection.BindingFlags.GetProperty,
                    null,
                    property,
                    new object[] { }).ToString();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception: {0}", ex.ToString());
            }
 
            return propValue;
        }


		/// <summary>
		/// Get source document. Open a FileDialog window for user to select file for
		/// parsing.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
        private void btnLoadSource_Click(object sender, System.EventArgs e)
		{
			if( openFileDialog.ShowDialog() == DialogResult.OK )
			{

				string	fileName = openFileDialog.FileName;
                String  outFileName = (string) fileName;

                outFileName = outFileName.ToLower().Replace(".ppt",".opml");

                DateTime modDate = File.GetLastWriteTime(fileName.ToString());
                DateTime createDate = File.GetCreationTime(fileName.ToString());

                // Date standard used in RFC822
                string dateFormat = "ddd dd MMM yyyy HH:mm:ss";
								
				object	readOnly	= false;
				object	visible		= true;				
  			    object	missing		= System.Reflection.Missing.Value;
                object  saveChanges = false;

                btnLoadSource.Text = "Loading document in PowerPoint...";
                btnLoadSource.Enabled = false;
                Application.DoEvents();
              
         		Microsoft.Office.Interop.PowerPoint.ApplicationClass appPPT = 
                    new Microsoft.Office.Interop.PowerPoint.ApplicationClass();

				// appPPT.Visible = Microsoft.Office.Core.MsoTriState.msoFalse;

				// Open the document
                Microsoft.Office.Interop.PowerPoint.Presentation docPPT = appPPT.Presentations.Open(fileName,
                    Microsoft.Office.Core.MsoTriState.msoFalse, Microsoft.Office.Core.MsoTriState.msoFalse,
                    Microsoft.Office.Core.MsoTriState.msoFalse);

                btnLoadSource.Text = "Converting document...";
                Application.DoEvents();

                // get document properties for use later
                object documentProperties = docPPT.BuiltInDocumentProperties;

                // Start writing the file
                outfil = new XmlTextWriter (outFileName, System.Text.Encoding.UTF8);

                progressBar1.Minimum = 1;
                progressBar1.Value = 1;
                progressBar1.Maximum = docPPT.Slides.Count;

                try
                {
                   outfil.Formatting = Formatting.Indented;
                   outfil.Indentation= 3;
                   outfil.Namespaces = false;

                   outfil.WriteStartDocument();

                   String instruction = "type=\"text/xsl\" href=\"/hyperscope/src/client/lib/hs/xslt/hyperscope.xsl\"";
                   outfil.WriteProcessingInstruction("xml-stylesheet", instruction);
       
                   outfil.WriteStartElement("opml");
                     outfil.WriteAttributeString("xmlns:hs", "http://www.hyperscope.org/hyperscope/opml/public/2006/05/09");
                     outfil.WriteAttributeString("hs:version", "1.0");
                     outfil.WriteAttributeString("version", "2.0");

                      outfil.WriteStartElement("head");
                        outfil.WriteAttributeString("hs:left-label-delim", "");
                        outfil.WriteAttributeString("hs:right-label-delim", "");
                        outfil.WriteAttributeString("hs:nidcount", docPPT.Slides.Count.ToString());
                        outfil.WriteElementString("ownerName", GetDocumentProperty(documentProperties, "Author") );
                        outfil.WriteElementString("dateCreated", createDate.ToString(dateFormat) + " GMT");
                        outfil.WriteElementString("dateModified", modDate.ToString(dateFormat) + " GMT");
                      outfil.WriteEndElement();   // Head

                      outfil.WriteStartElement("body");

                      // Go slide by slide...
                      for (int slideNum = 1; slideNum <= docPPT.Slides.Count; slideNum++)
                      {

                          progressBar1.Value = slideNum;
                          Application.DoEvents();

                          Microsoft.Office.Interop.PowerPoint.Slide sld = docPPT.Slides[slideNum];
                          string title = GetTitle(sld, slideNum);

                          outfil.WriteStartElement("outline");
                          outfil.WriteAttributeString("text", "<h1>"+title+"</h1>");
                          outfil.WriteAttributeString("hs:nid", "0" + sld.SlideID.ToString());

                          // For each placeholder (except first one which is title), recursively
                          //   next outline elements by calling writeSection
                          foreach (Microsoft.Office.Interop.PowerPoint.Shape shp in sld.Shapes.Placeholders)
                          {
                              if ((shp != sld.Shapes.Title) && (shp.HasTextFrame == Microsoft.Office.Core.MsoTriState.msoTrue))
                              {
                                  parNum = 1;
                                  parNums = shp.TextFrame.TextRange.Paragraphs(0, 30000).Count;
                                  while (parNum <= parNums)
                                  {
                                      writeSection(shp, sld.SlideID);
                                  }
                              }
                          }

                          // Include notes
                          if (mnuIncludeNotes.Checked)
                          {
                              // See if there are any notes
                              string notes = "";
                              foreach (Microsoft.Office.Interop.PowerPoint.Shape noteShape in sld.NotesPage.Shapes)
                                  if (noteShape.HasTextFrame == Microsoft.Office.Core.MsoTriState.msoTrue)
                                      notes = notes + CleanString(noteShape.TextFrame.TextRange.Text.Trim());

                              // Notes of a slide are a slide!  Loop through all paragraphs and create a node for each.
                              if (notes != "")
                              {
                                  parNum = parNum + 1;
                                  outfil.WriteStartElement("outline");
                                  outfil.WriteAttributeString("text", "<H2>NOTES</H2>");
                                  outfil.WriteAttributeString("hs:nid", "0" + sld.SlideID.ToString() + parNum.ToString());

                                  foreach (Microsoft.Office.Interop.PowerPoint.Shape noteShape in sld.NotesPage.Shapes)
                                      if (noteShape.HasTextFrame == Microsoft.Office.Core.MsoTriState.msoTrue)
                                          foreach (Microsoft.Office.Interop.PowerPoint.TextRange tr in noteShape.TextFrame.TextRange.Paragraphs(0, 30000))
                                          {
                                              parNum = parNum + 1;
                                              outfil.WriteStartElement("outline");
                                              outfil.WriteAttributeString("text", "<H3>" + CleanString(tr.Text.Trim()) + "</H3>");
                                              outfil.WriteAttributeString("hs:nid", "0" + sld.SlideID.ToString() + parNum.ToString());
                                              outfil.WriteEndElement();
                                          }
                                  outfil.WriteEndElement();  // NOTES

                              }
                          }

                          // Include picture
                          if (mnuSaveJPGs.Checked)
                          {
                              string fullSlideName = outFileName.ToLower().Replace(".opml", "-" + sld.SlideID.ToString() + ".jpg");
                              string slideName = fullSlideName.Substring(fullSlideName.LastIndexOf("\\") + 1);
                              sld.Export(fullSlideName, "JPG", 800, 600);
                              parNum = parNum + 1;
                              outfil.WriteStartElement("outline");
                              outfil.WriteAttributeString("text", "<H2>SLIDE IMAGE</H2>");
                              outfil.WriteAttributeString("hs:nid", "0" + sld.SlideID.ToString() + parNum.ToString());
                                 parNum = parNum + 1;
                                 outfil.WriteStartElement("outline");
                                 outfil.WriteAttributeString("text", "<img src=\"" +slideName + "\" />");
                                 outfil.WriteAttributeString("hs:nid", "0" + sld.SlideID.ToString() + parNum.ToString());
                                 outfil.WriteEndElement();
                              outfil.WriteEndElement();
                          }
                          outfil.WriteEndElement(); // Slide

                      }
                   
                      outfil.WriteEndElement();  // Body
                      outfil.WriteEndElement();  // OPML
                   outfil.WriteEndDocument();

                   outfil.Flush();
                }
                catch(Exception ex)
                {
                    Console.WriteLine("Exception: {0}", ex.ToString());
                }
                finally
                {
                    if (outfil != null)
                    {
                        outfil.Close();
                        progressBar1.Value = 1;
                        btnLoadSource.Enabled = true;
                        btnLoadSource.Text = "Load PPT File";
                        Application.DoEvents();
                    }
                    // close word application
                    appPPT.Quit();
                }
			}
		}

        /// <summary>
        /// Exit the program
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
		private void mnuExit_Click(object sender, System.EventArgs e)
		{
			Close();	
		}

        private void mnuAbout_Click(object sender, EventArgs e)
        {
            frmAboutBox frmAbout = new frmAboutBox();
            frmAbout.ShowDialog();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {

        }

        private void mnuSaveJPGs_Click(object sender, EventArgs e)
        {
            mnuSaveJPGs.Checked = !mnuSaveJPGs.Checked;
        }

        private void mnuIncludeNotes_Click(object sender, EventArgs e)
        {
            mnuIncludeNotes.Checked = !mnuIncludeNotes.Checked;
 
        }


	}
}
